#ifndef _ACTION_H
#define _ACTION_H

#include <pthread.h>
#include <client_handler.h>
#include "object.h"

class Druid;

class Background {
    static const int max_count = 3;
    Druid& act;
    double displace;
    double last;
    double period;
    double maximal;
    double same;
    double value;
    unsigned int errcount;
    bool to_check;
public:
    Background( Config&, Druid& );
    void set_checking( bool ch) { to_check = ch ;}
    bool run( bool );
    bool run();
    bool no_needed();
    bool needed();
    double get() { return value; }
private:
    double measure();
};

class Druid {
    static const int max_attempts = 8;
    static const int max_retries = 2;
    std::auto_ptr<ClientHandler> dimm;
    std::auto_ptr<ClientHandler> mass;
    std::auto_ptr<ClientHandler> tlsp;
    Background background;
    unsigned int settle_time;
    unsigned int errcount;
    double focus;
    double separation;
    double sep_tolrn;
    double work_zone;
    double mass_mag_ext;
    double star_flux_d;
    unsigned int errflux;
    std::vector<double> mass_mag_set;
    friend class Background;
    bool mass_enable;
    bool cnt_on;
    double guide_last;
public:
    Druid( Config& );
    ~Druid();
    bool point_star( const ObjectInfo& );
    bool check_star( const ObjectInfo& );
    bool check_status( const ObjectInfo& );
    bool check_experiment_status( const ObjectInfo& );
    bool check_background();
    bool check_flux();
    bool measurement();
    bool dimm_measurement();
    bool beginning();
    bool photo_stability( unsigned int );
    bool cnt( bool );
    void regime( const std::string& );
    void check_background_on() { background.set_checking(true); }
    void check_background_off() { background.set_checking(false); }
private:
    bool center_star(  const ObjectInfo& );
    bool init();
    bool park( ClientHandler* );
    void focusing( double );
    bool focus_dimm();
    double check_separation();
    bool stop_guiding();
    bool start_guiding();
    bool start_dimm_guiding();
    bool check_mass_mag( const ObjectInfo&, double );
    void camera( const std::string& );
};

#endif
